/*! \file ./includes/GateToASCII.hh

    $Log: GateToASCII.hh,v $
    Revision 1.14  2004/09/16 14:00:21  gsantin
    FileSizeLimit and Coinc/Single masks

    Revision 1.13  2003/11/12 11:10:22  dstrul
    Bug fix: corrected problem with setting of output file name for Singles and Coincidences files

    Revision 1.12  2003/07/02 22:19:44  dstrul
    Digitizer re-engineering

    Revision 1.11  2003/06/21 12:01:23  dstrul
    Output modules now handle multiple digi channels

    Revision 1.10  2002/11/24 15:10:14  gsantin
    new RecordVoxels, with disabling flag

    Revision 1.9  2002/11/23 22:51:27  gsantin
    RecordVoxels in the output modules, Root 3D Histo with the voxel densities

    Revision 1.8  2002/10/21 15:23:29  dstrul
    Adaptations for DigiGate: simplified the use of the compilation options, added security for the case when the user tries to run DigiGate without the ROOT option

    Revision 1.7  2002/10/19 14:53:39  dstrul
    Adaptations for Digigate: the opening and closinf of the output files has been transfered into the new methods RecordBeginOfAcquisition() and RecordEndOfAcquisition(); the name of the output file can be set by the user

    Revision 1.6  2002/10/19 13:15:51  dstrul
    Adaptations for DigiGate: the hit data are no longer written when in off-line digitation mode

    Revision 1.5  2002/10/19 10:50:11  dstrul
    Adaptations for DigiGate: the retrieving of the hits and digis collections is now handled by methods provided by the base-class GateVOutputModule

    Revision 1.4  2002/08/11 15:33:24  dstrul
    Cosmetic cleanup: standardized file comments for cleaner doxygen output

    Revision 1.3  2002/05/13 12:51:41  gsantin
    documentation added

        
    - GateToASCII - by Delphine Lazaro <lazaro@clermont.in2p3.fr> 
    - Modifications 
      G.Santin   3.2002 for GATE
      G.Santin 1.5.2002 GateVOutputModule structure

    - It is an Output Module, i.e. called by the Gate Output Manager. 
      Its function is to do an ASCII output of Hits and Digis (Singles and Coincidences).
      It uses the operators << defined in the Hit and Digis files
      
    - It is derived from GateVOutputModules, that provides tha base skeleton
      of all the Output classes, both for this Mgr and for the Output Modules.

    - It has a Messenger to set the flags for the file writing

   \sa GateVOutputModule
   \sa GateOutputModuleMessenger
   \sa GateToRoot
   \sa GateToLMF

  \brief Class GateToASCII
  \brief By lazaro@clermont.in2p3.fr
  \brief $Id: GateToASCII.hh,v 1.14 2004/09/16 14:00:21 gsantin Exp $
*/

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

#ifndef GateToASCII_H
#define GateToASCII_H

#ifdef G4ANALYSIS_USE_FILE

#include "GateVOutputModule.hh"

class GateToASCIIMessenger;

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

class GateToASCII :  public GateVOutputModule
{
public:
  GateToASCII(const G4String& name, GateOutputMgr* outputMgr,DigiMode digiMode);

  virtual ~GateToASCII();

  //! It opens the ASCII files
  void RecordBeginOfAcquisition();
  //! It closes the ASCII files.
  void RecordEndOfAcquisition();
  void RecordBeginOfRun(const G4Run *);
  //! It saves the run-specific info in the Run file
  void RecordEndOfRun(const G4Run *);
  void RecordBeginOfEvent(const G4Event *);
  //! saves the Hits in the ASCII files, and calls RecordDigitizer
  void RecordEndOfEvent(const G4Event *);
  void RecordStep(const G4Step *);
  //! saves the geometry voxel information
  void RecordVoxels(GateVGeometryVoxelStore *);

  //! Called by RecordEndOfEvent to store the Digis in the ASCII files
  void RecordDigitizer(const G4Event *);

  //! Nothing
  void Reset();

  class VOutputChannel
  {
    public:
      inline VOutputChannel(const G4String& aCollectionName,
			    G4bool outputFlag)
        : m_verboseLevel(0),
	  m_outputFlag(outputFlag),
  	  m_collectionName(aCollectionName),
	  m_collectionID(-1),
	  m_fileCounter(0),
	  //	  m_outputFileSizeLimit(2000000000),
	  m_fileBaseName(G4String(""))
	{} 
      virtual inline ~VOutputChannel() {}
      
      void Open(const G4String& aFileBaseName);
      void Close();
      static void SetOutputFileSizeLimit(G4int limit) {m_outputFileSizeLimit = limit;};
      G4bool ExceedsSize();
      virtual void RecordDigitizer()=0;
      
      inline void SetOutputFlag(G4bool flag) { m_outputFlag = flag; };
      inline void SetVerboseLevel(G4int val) { m_verboseLevel = val; };

      
      G4int             m_verboseLevel;
      G4bool            m_outputFlag;
      G4String          m_fileBaseName;
      G4String          m_collectionName;
      G4int             m_fileCounter;
      long              m_outputFileBegin;
      G4int	        m_collectionID;
      G4std::ofstream   m_outputFile;

      static long       m_outputFileSizeLimit;
  };
  
  class SingleOutputChannel : public VOutputChannel
  {
    public:
      SingleOutputChannel(const G4String& aCollectionName,
			       G4bool outputFlag);
      virtual inline ~SingleOutputChannel() {}
      virtual void RecordDigitizer();
  };
  
  class CoincidenceOutputChannel : public VOutputChannel
  {
    public:
      CoincidenceOutputChannel(const G4String& aCollectionName,
			       G4bool outputFlag);
      virtual inline ~CoincidenceOutputChannel() {}
      virtual void RecordDigitizer();
  };
  
  //! flag to decide if it writes or not to the file
  G4bool GetOutFileRunsFlag()           { return m_outFileRunsFlag; };
  //! flag to decide if it writes or not to the file
  void   SetOutFileRunsFlag(G4bool flag) { m_outFileRunsFlag = flag; };

  //! flag to decide if it writes or not to the file
  G4bool GetOutFileHitsFlag()           { return m_outFileHitsFlag; };
  //! flag to decide if it writes or not to the file
  void   SetOutFileHitsFlag(G4bool flag) { m_outFileHitsFlag = flag; };

  //! flag to decide if it writes or not to the file
  G4bool GetOutFileVoxelFlag()               { return m_outFileVoxelFlag; };
  //! flag to decide if it writes or not to the file
  void   SetOutFileVoxelFlag(G4bool flag)    { m_outFileVoxelFlag = flag; };

  //! Get the output file name
  const  G4String& GetFileName()             { return m_fileName; };
  //! Set the output file name
  void   SetFileName(const G4String aName)   { m_fileName = aName; };

  void   RegisterNewCoincidenceDigiCollection(const G4String& aCollectionName,G4bool outputFlag);
  void   RegisterNewSingleDigiCollection(const G4String& aCollectionName,G4bool outputFlag);

  void SetVerboseLevel(G4int val) 
  { 
    GateVOutputModule::SetVerboseLevel(val);
    for (size_t i=0; i<m_outputChannelList.size(); ++i)
      m_outputChannelList[i]->SetVerboseLevel(val);
  };

private:

  G4bool   m_outFileRunsFlag;
  G4bool   m_outFileHitsFlag;
  G4bool   m_outFileVoxelFlag;


  GateToASCIIMessenger* m_asciiMessenger;

  G4std::ofstream m_outFileRun;
  G4std::ofstream m_outFileHits;

  G4String m_fileName;

  G4std::vector<VOutputChannel*>  m_outputChannelList;
};

#endif
#endif
